@@ -358,8 +358,10 @@ U16 determineParameterLength( U16 sprm, const U8* in, WordVersion version )
 // Apply a <T> grpprl of a given size ("count" bytes long)
 // This template function might be a bit sick, but it helps to
 // avoid duplicated code, so what ;)
-template<class T, S16 ( T::* applySPRM ) ( const U8*, const Style*, OLEStreamReader*, WordVersion )>
-void apply( T* const t, const U8* grpprl, U16 count, const Style* style, OLEStreamReader* dataStream, WordVersion version )
+template<class T>
+void apply(T* const t,
+           S16 ( T::* applySPRM ) ( const U8*, const Style*, OLEStreamReader*, WordVersion ),
+           const U8* grpprl, U16 count, const Style* style, OLEStreamReader* dataStream, WordVersion version )
 {
     if ( !grpprl )
         return;
@@ -568,7 +570,7 @@ Word97::TAP* initTAP( const U8* exceptions, OLEStreamReader* dataStream, WordVer
 void PAP::apply( const U8* grpprl, U16 count, const Style* style, OLEStreamReader* dataStream, WordVersion version )
 {
     // A PAP grpprl might contain TAP sprms, we just skip them
-    SPRM::apply<PAP, &PAP::applyPAPSPRM>( this, grpprl, count, style, dataStream, version );
+    SPRM::apply<PAP>( this, &PAP::applyPAPSPRM, grpprl, count, style, dataStream, version );
 }
 
 // Helper methods for the more complex sprms
@@ -577,11 +579,12 @@ namespace
     // Adds the tabs of the sprmPChgTabs* sprms. Pass a pointer to the
     // itbdAddMax and the vector
     // Returns the number of tabs added
-    U8 addTabs( const U8* ptr, std::vector<Word97::TabDescriptor>& rgdxaTab )
+    typedef std::vector<Word97::TabDescriptor> TabDescVector;
+    U8 addTabs( const U8* ptr, TabDescVector& rgdxaTab )
     {
         //wvlog << "Before adding the tabs: " << (int)rgdxaTab.size() << std::endl;
         // Remember where the end was
-        const std::vector<Word97::TabDescriptor>::size_type oldSize = rgdxaTab.size();
+        const TabDescVector::size_type oldSize = rgdxaTab.size();
         // Now append the new ones, we'll then sort the vector using inplace_merge
         const U8 itbdAddMax = *ptr++;
         //wvlog << "                           itbdAddMax=" << (int)itbdAddMax << std::endl;
@@ -594,12 +597,14 @@ namespace
             descr.tbd = TBD( readU8( ptr + sizeof( S16 ) * itbdAddMax + i ) );
             rgdxaTab.push_back( descr );
         }
-        if (oldSize != 0) {
-            std::vector<Word97::TabDescriptor>::iterator middle = rgdxaTab.begin();
-            middle += oldSize + 1;
+        if ( oldSize != 0 && itbdAddMax ) {
+            TabDescVector::iterator middle = rgdxaTab.begin();
+            middle += oldSize + 1u;
             std::inplace_merge( rgdxaTab.begin(), middle, rgdxaTab.end() );
         }
-        std::unique( rgdxaTab.begin(), rgdxaTab.end() );
+        TabDescVector::iterator uend = std::unique( rgdxaTab.begin(), rgdxaTab.end() );
+        if ( uend != rgdxaTab.end() )
+            rgdxaTab.erase( uend, rgdxaTab.end() );
         //wvlog << "After applying sprmPChgTabs(Papx) : " << (int)rgdxaTab.size() << std::endl;
         return itbdAddMax;
     }
@@ -999,7 +1004,7 @@ S16 PAP::applyPAPSPRM( const U8* ptr, const Style* /*style*/, OLEStreamReader* d
 void CHP::apply( const U8* grpprl, U16 count, const Style* paragraphStyle, OLEStreamReader* dataStream, WordVersion version )
 {
     // There should be only CHP sprms in the grpprl we get
-    SPRM::apply<CHP, &CHP::applyCHPSPRM>( this, grpprl, count, paragraphStyle, dataStream, version );
+    SPRM::apply<CHP>( this, &CHP::applyCHPSPRM, grpprl, count, paragraphStyle, dataStream, version );
 }
 
 void CHP::applyExceptions( const U8* exceptions, const Style* paragraphStyle, OLEStreamReader* dataStream, WordVersion version )
@@ -1457,7 +1462,7 @@ S16 CHP::applyCHPSPRM( const U8* ptr, const Style* paragraphStyle, OLEStreamRead
 void PICF::apply( const U8* grpprl, U16 count, const Style* style, OLEStreamReader* dataStream, WordVersion version )
 {
     // There should be only PICF sprms in the grpprl we get
-    SPRM::apply<PICF, &PICF::applyPICFSPRM>( this, grpprl, count, style, dataStream, version );
+    SPRM::apply<PICF>( this, &PICF::applyPICFSPRM, grpprl, count, style, dataStream, version );
 }
 
 void PICF::applyExceptions(const U8* /*exceptions*/, const StyleSheet* /*stylesheet*/, OLEStreamReader* /*dataStream*/, WordVersion /*version*/ )
@@ -1520,7 +1525,7 @@ S16 PICF::applyPICFSPRM( const U8* ptr, const Style* /*style*/, OLEStreamReader*
 void SEP::apply( const U8* grpprl, U16 count, const Style* style, OLEStreamReader* dataStream, WordVersion version )
 {
     // There should be only SEP sprms in the grpprl we get
-    SPRM::apply<SEP, &SEP::applySEPSPRM>( this, grpprl, count, style, dataStream, version );
+    SPRM::apply<SEP>( this, &SEP::applySEPSPRM, grpprl, count, style, dataStream, version );
 }
 
 void SEP::applyExceptions( const U8* exceptions, const StyleSheet* /*stylesheet*/, OLEStreamReader* dataStream, WordVersion version )
@@ -1733,7 +1738,7 @@ void TAP::apply( const U8* grpprl, U16 count, const Style* style, OLEStreamReade
 {
     // There should be mostly TAP sprms in the grpprl we get, and we
     // have to ignore the remaining PAP sprms, just what the template does
-    SPRM::apply<TAP, &TAP::applyTAPSPRM>( this, grpprl, count, style, dataStream, version );
+    SPRM::apply<TAP>( this, &TAP::applyTAPSPRM, grpprl, count, style, dataStream, version );
 }
 
 void TAP::applyExceptions( const U8* /*exceptions*/, const StyleSheet* /*stylesheet*/, OLEStreamReader* /*dataStream*/, WordVersion /*version*/ )
